from graphviz import Graph
from graphviz import Digraph
import numpy as np

from random import randint, random
from matplotlib import pyplot as plt

###génération de divers types de matrices pour divers types de réseaux
##réseaux de type aléatoire pour un graphe connexe
def mat_connexe(n, p=0.2):
    """génère une matrice n*n avec des 0 et des 1 , 0 sur la diagonale, au moins un 1 par ligne, une probabilité p d'avoir 1"""
    m=np.full(shape=(n, n),fill_value=0)#initialise un matrice n*n avec des 0
    for i in range(n):#ligne
        for j in range(n):#colonne
            if i != j:
                m[i][j] = int(random() < p)# 1 en position (i,j) avec un probabilité de p
            else:
                m[i][j] = 0#0 sinon
    for i in range(n-1):
        if sum(m[i]) == 0:#pour ajouter un 1 au cas où la ligne ne comporte pas du tout de 1
            m[i][randint(i+1, n-1)]=1#on place un 1 aléatoirement après la diagonale
    return m

def mat_connexe_sym(n, p=0.2):
    """génère une matrice SYMETRIQUE n*n avec des 0 et des 1 , 0 sur la diagonale, au moins un 1 par ligne, une probabilité p d'avoir 1"""
    m=np.full(shape=(n, n),fill_value=0)#initialise un matrice n*n avec des 0
    for i in range(n):#ligne
        for j in range(i+1,n):#colonne pour une matrice triangulaire supérieure
            m[i][j] = int(random() < p)# 1 en position (i,j) avec un probabilité de p
    for i in range(n-1):#pour ajouter un 1 au cas où
        if sum(m[i]) == 0:
            m[i][randint(i+1, n-1)]=1
    return m+np.transpose(m)#on retourne bien une matrice symétrique
    
##réseau où tout le monde est relié (orienté / non orienté)
def mat_tout_o(n):
    mat=np.array([])
    l=[]
    for i in range(n):
        for j in range(n):
            if i == j:
                coef=0
            else:
                coef=1 
            l.append(coef)
    mat=np.asarray(l)
    mat.resize(n,n)
    return mat
  

def mat_tout_no(n):
    mat=np.array([])
    l=[]
    for i in range(n):
        for j in range(n):
            if i >= j:
                coef=0
            else:
                coef=1 
            l.append(coef)
    mat=np.asarray(l)
    mat.resize(n,n)
    return mat
    
##réseau type serveur (orienté / non orienté)   
def mat_serveur_o(n):#orienté
    mat=np.array([])
    l=[]
    for i in range(n):
        for j in range(n):
            if i == 0 and i == j:
                coef=0
            elif i == 0 :
                coef=1 
            elif j == 0:
                coef=1
            else :
                coef=0
            l.append(coef)
    mat=np.asarray(l)
    mat.resize(n,n)
    return mat
    
def mat_serveur_no(n):#non orienté
    m=np.full(shape=(n, n),fill_value=0)#initialise un matrice n*n avec des 0
    for i in range(1,n):
        m[0][i]=1#on met 1 sur la 1ere ligne hors diagonale
    return m+np.transpose(m)
    
#concatener deux matrices pour modéliser plusieurs réseaux disjoints
def conca(mat1, mat2):
    li=[]
    n1=mat1.shape[0] #taille de la matrice (carrée) mat1
    n2=mat2.shape[0]
    for i in range(n1):
        for j in range(n1):
            li.append(mat1[i][j])
        for j in range(n1+1, n1+n2+1):
            li.append(0)
    for i in range(n1+1, n1+n2+1):
        for j in range(n1):
            li.append(0)
        for j in range(n1+1, n1+n2+1):
            li.append(mat2[i-n1-1][j-n1-1])        
    mat=np.asarray(li)
    mat.resize(n1+n2,n1+n2)
    return mat
    
#concatener n matrices (sans en connaître le nombre a priori, n >=2 !!)
def conca_plus(*args):#les paramètres doivent être des matrices, au moins deux)
    res = conca(args[0],args[1])
    if len(args) == 2:
        return res
    else:
        for i in range(2,len(args)):
            res = conca(res, args[i])
        return res

#ajouter quelques liens au hasard entre deux réseaux disjoints
def lien(M):#ajoute un lien si on tombe sur un 0 ; sinon ne fait rien
    """peut ajouter un 1 dans une matrice comprenant de nombreux zéros"""
    N = M.shape[0]-1
    I = randint(0, N)
    J = randint(0, N)
    if I != J and M[I][J] == 0:
        M[I][J] = 1
    return M
#repète rep fois la fonction précédente
def lien_n(M, rep):#n'ajoute pas forcément n liens ... car on peut tomber sur des 1
    for i in range(rep):
        M = lien(M)
    return M

##création du graphe à partir de la matrice considérée
#un graphe orienté prendra des matrices quelconques
#pour un graphe non orienté, prendre obligatoirement une matrice symétrique
def graph(mat, type, v=True, p=False):
    """matrice à visualiser ; type = 1 pour orienté, type = 2 pour non orienté ; v = view (True par défaut) ; p pour voir la matrice dans la console (False par défaut)"""
    g = Graph(format='png')
    if type == 1:#graphe orienté
        g = Digraph('G', filename='mat_graph', engine='sfdp')
        for i in range(mat.shape[0]):
            g.node('S'+str(i))
            for j in range(mat.shape[1]):
                if mat[i][j] == 1 :
                    g.edge('S'+str(i), 'S'+str(j))
        g.render("math_graph", view=v) 
        if p:
            print(mat)
    elif type == 2:#graphe non orienté
        g = Graph('G', filename='mat_graph', engine='sfdp')
        for i in range(mat.shape[0]):
            g.node('S'+str(i))
            for j in range(mat.shape[1]):
                if j > i and mat[i][j] == 1 : #on ne s'occupe que des termes de la diagonale supérieure
                    g.edge('S'+str(i), 'S'+str(j))
        g.render("math_graph", view=v) 
        if p:
            print(mat)
    else:
        print("type = 1 pour orienté ; ou type = 2 pour non orienté")

def graph2(mat, type=1, v=True, p=False):#version simplifiée
    """matrice à visualiser ; type = 1 pour orienté, type = 2 pour non orienté ; v = view (True par défaut) ; p pour voir la matrice dans la console (False par défaut)"""
    g = Graph(format='png')
    nom=['A','B','C']
    if type == 1:#graphe orienté
        g = Digraph('G', filename='mat_graph', engine='sfdp')
        for i in range(mat.shape[0]):
            g.node('P'+str(i+1))
            for j in range(mat.shape[1]):
                if mat[i][j] == 1 :
                    g.edge('P'+str(i+1), 'P'+str(j+1))
        g.render("math_graph", view=v)